@@ -127,11 +127,12 @@ extern SEXP rnng_close(SEXP);
 extern SEXP rnng_ctx_close(SEXP);
 extern SEXP rnng_ctx_open(SEXP);
 extern SEXP rnng_device(SEXP, SEXP);
-extern SEXP rnng_dial(SEXP, SEXP, SEXP);
+extern SEXP rnng_dial(SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_dialer_close(SEXP);
 extern SEXP rnng_dialer_start(SEXP, SEXP);
 extern SEXP rnng_get_opt(SEXP, SEXP);
-extern SEXP rnng_listen(SEXP, SEXP, SEXP);
+extern SEXP rnng_is_nul_byte(SEXP);
+extern SEXP rnng_listen(SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_listener_close(SEXP);
 extern SEXP rnng_listener_start(SEXP);
 extern SEXP rnng_messenger(SEXP);
@@ -153,6 +154,7 @@ extern SEXP rnng_sha256(SEXP, SEXP, SEXP);
 extern SEXP rnng_sha384(SEXP, SEXP, SEXP);
 extern SEXP rnng_sha512(SEXP, SEXP, SEXP);
 extern SEXP rnng_sleep(SEXP);
+extern SEXP rnng_stats_get(SEXP, SEXP);
 extern SEXP rnng_status_code(SEXP);
 extern SEXP rnng_stream_close(SEXP);
 extern SEXP rnng_stream_dial(SEXP, SEXP, SEXP);
@@ -161,6 +163,7 @@ extern SEXP rnng_strerror(SEXP);
 extern SEXP rnng_subscribe(SEXP, SEXP, SEXP);
 extern SEXP rnng_thread_create(SEXP);
 extern SEXP rnng_unresolved(SEXP);
+extern SEXP rnng_url_parse(SEXP);
 extern SEXP rnng_version(void);
 
 #endif
